-- EGAify takes an image and finds the blended EGA equivalent.
-- Author: Dan Beeston - https://invisiblespiders.com
-- Twitter: @dnabeast
-- It aims to emulate the Sierra SCI0 style enhanced EGA palette

-- INSTALLATION
-- Open Aseprite. Select File -> scripts -> open scripts folder.
-- Copy this file into that folder and restart Aseprite.
-- It's recommended to assign a hot key to the script. I use Alt-e

-- CONFIG

-- Some dither combinations are garish and have very close alt colours
-- if you want to return these combinations change this flag to false
-- (Useful for hi res images with lots of subtle gradiations.)
removeGarishCombinations = true


-- function to blend 2 colors together using lab color
local function lrgb(col1, col2, f)
	local x1 = col1.red
	local y1 = col1.green
	local z1 = col1.blue
	local x2 = col2.red;
	local y2 = col2.green;
	local z2 = col2.blue;

	return Color({
		r=(math.sqrt((x1^2) * (1-f) + (x2^2) * f)),
		g=(math.sqrt((y1^2) * (1-f) + (y2^2) * f)),
		b=(math.sqrt((z1^2) * (1-f) + (z2^2) * f)),
	})
end

-- We need the sprite loaded so that we can add the palette later.
local spr = app.activeSprite
if not spr then
  return app.alert("There is no active sprite")
end

local ega = Palette{ fromResource="CGA" }

-- Create the Palette we plan to use
local palette = Palette(16*16)

local badColors = {
33,65,66,67,82,98,99,113,164,165,166,180,182,194,195,202,203,218,233,210,248,228,229,230
}

-- Hydrate the palette
for index = 0, #palette-1 do
	-- Don't place duplicate colours
	if (index%16 > math.floor(index/16)) then
		palette:setColor(index, Color({r=0,g=0,b=0,a=255}))
	else
		palette:setColor(index, lrgb( ega:getColor(math.floor(index/16)), ega:getColor(index%16), 0.5) )
	end

	-- remove ugly combinations
	if (removeGarishCombinations == true) then
		for key, color in pairs(badColors) do
			if (index == color) then
				palette:setColor(index, Color({r=0,g=0,b=0,a=255}))
			end
		end
	end

end

-- this is wrapped in a transaction so that the flattening and palette switching
-- is a separate undo to the EGA dithering
app.transaction(
	function()
		-- we make sure we're in RGB mode so that the palette change doesn't shuffle our image colors
		app.command.ChangePixelFormat{ format="rgb", dithering="none" }
		spr:setPalette(palette)
		-- we flatten our layers so the effect happens to the whole image
		app.command.FlattenLayers()
		app.command.BackgroundFromLayer()
		-- we switch to an indexed palette to make them easier to work with
		app.command.ChangePixelFormat{ format="indexed", dithering="none" }
	end
)

local cel = app.activeCel
if not cel then
  return app.alert("There is no active image")
end

-- We clone the image to that the dithering is a single undo.
local img = cel.image:clone()

	for pixel in img:pixels() do
		-- for each pixel we get the color
		local col = img:getPixel(pixel.x, pixel.y)
		local primaryMixer = ega:getColor(math.floor(col/16))
		local secondaryMixer = ega:getColor(math.floor(col%16))

		-- here we make sure the brighter mixer is always in the same quadrant of the checker.
		if (primaryMixer.value >= secondaryMixer.value) then
			local x = secondaryMixer
			secondaryMixer = primaryMixer
			primaryMixer = x
		end

		--place the mixer colors based on the position in the checker.
		if (((pixel.x+pixel.y)%2) == 1) then
			img:drawPixel(pixel.x, pixel.y, primaryMixer)
		else
			img:drawPixel(pixel.x, pixel.y, secondaryMixer)
		end

	end

-- set the image
cel.image = img

app.refresh()

